diff --git a/drivers/misc/hi6220-sysconfig.c b/drivers/misc/hi6220-sysconfig.c
new file mode 100644
index 000000000000..0d6af3af90ba
--- /dev/null
+++ b/drivers/misc/hi6220-sysconfig.c
@@ -0,0 +1,70 @@
+#include <linux/io.h>
+ 
+#define SOC_HI6220_ACPU_SCTRL_BASE_ADDR 0xF6504000
+#define ACPU_SC_CLKEN 0x00C
+#define ACPU_SC_CLK_STAT 0x014
+
+// Heavily inspired by
+// https://github.com/96boards-hikey/arm-trusted-firmware/commit/543f1db7fd7e0aadcf947adedd63eaed817ef402
+static int ensure_debug_mode_on(void __iomem *base, unsigned int cpu,
+				unsigned int cluster)
+{
+	void __iomem *base_off;
+	unsigned int data, expected;
+	// Some number high enough to basically never fail, but also to not lock
+	// up the system in case my code is broken.
+	const unsigned int max_loop_retries = 10000;
+	unsigned int loop_retries = 0;
+
+	base_off = base + 0x02c;
+	data = readl_relaxed(base_off);
+	if (cluster)
+		expected = 1 << (cpu + 8);
+	else
+		expected = 1 << cpu;
+	writel_relaxed((data | expected), base_off);
+
+	for (loop_retries = 0; loop_retries < max_loop_retries;
+	     loop_retries++) {
+		/* RAW barrier */
+		data = readl_relaxed(base_off);
+		if (data & expected)
+			break;
+	}
+	return loop_retries == max_loop_retries;
+}
+
+
+static int __init hi6220_sysconf(void)
+{
+	void __iomem *base = NULL;
+	unsigned int cpu, cluster;
+
+	base = ioremap(SOC_HI6220_ACPU_SCTRL_BASE_ADDR, SZ_4K);
+	if (base == NULL) {
+		pr_err("hi6220: asctl reg iomap failed!\n");
+		return -ENOMEM;
+	}
+	/* enable coresight */
+	pr_err("%s:before sctrl ACPU_SC_CLK_STAT is %x\n",
+	       __func__,
+	       readl(base + ACPU_SC_CLK_STAT));
+	writel(BIT(11), base + ACPU_SC_CLKEN);
+	pr_err("%s:after sctrl ACPU_SC_CLK_STAT is %x\n",
+	       __func__, readl(base + ACPU_SC_CLK_STAT));
+
+	for (cluster = 0; cluster < 2; cluster++) {
+		for (cpu = 0; cpu < 4; cpu++) {
+			if (ensure_debug_mode_on(base, cpu, cluster)) {
+				pr_err("hi6220: flipping debug mode for cpu %d "
+				       "failed.\n", cpu + cluster*4);
+				// Keep trying for the remainder anyway.
+			}
+		}
+	}
+
+	iounmap(base);
+
+	return 0;
+ }
+ postcore_initcall(hi6220_sysconf);
